@@ -1,5 +1,5 @@
 /*
- MRTG 2.14.3  -- Rateup
+ MRTG 2.14.4  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -83,7 +83,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.14.3";
+char *VERSION = "2.14.4";
 char *program, *router, *routerpath;
 int histvalid;
 
@@ -109,7 +109,8 @@ struct tm * stLocal;
 time_t timestamp;
 char bufftime[32];
 
-char *short_si[] = { "", "k", "M", "G", "T" };
+char *short_si_def[] = { "", "k", "M", "G", "T" };
+char **short_si = short_si_def;
 char *longup = NULL;
 char *shortup = NULL;
 char *pngtitle = NULL;
@@ -152,7 +153,6 @@ int col_outp[3];
 
 long long kilo = (long long) 1000;
 char *kMG = (char *) NULL;
-char *kMGcopy = (char *) NULL;	/* JML bugfix */
 int kMGnumber = 0;
 
 #define MAXL	200		/* Maximum length of last in & out fields */
@@ -244,6 +244,20 @@ for debuging
     }\
   }
 
+static double logscale(double y, double maxy)
+{
+     y = (y * (maxy - 1) / maxy) + 1;
+     y = log(y) / log (maxy) * maxy;
+     if (y < 0) return 0;
+     if (y > maxy) return maxy;
+     return y;
+}
+
+static double expscale(double y, double maxy)
+{
+    y = exp(y / maxy * log(maxy));
+    return (y - 1) * maxy / (maxy - 1);
+}
 
 static void
 image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
@@ -275,7 +289,7 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
   char ylab[30];
   /* scaling helpers */
   long long maxv_q;
-  long long valsamp, maxin, maxout, digits, maxpercent = 0;
+  long long valsamp, maxin, maxout, digits, digits1, maxpercent = 0;
   long long sca_ten, sca_hun;
   double nmax_q;
   double avmxin, avmxout, avin, avout, latestout = 0, latestin =
@@ -461,12 +475,25 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 
   if (kMG)
     {
-      char *string = kMG;
-      kMGnumber = 0;
-      while (strchr (string, ','))
-	{
-	  string = strchr (string, ',') + 1;
-	  kMGnumber++;
+      if (short_si[0] != kMG)
+        {
+          short_si_out = kMG;
+          kMGnumber = 0;
+          while ((short_si_out = strchr (short_si_out, ',')) != NULL)
+	    {
+	      short_si_out++;
+	      kMGnumber++;
+	    }
+
+          short_si = calloc(kMGnumber + 1, sizeof(*short_si));
+          short_si_out = kMG;
+          for (kMGnumber = 0; ; kMGnumber++)
+            {
+	      short_si[kMGnumber] = short_si_out;
+	      short_si_out = strchr(short_si_out, ',');
+	      if (short_si_out == NULL) break;
+	      short_si_out++;
+	    }
 	}
     }
   else
@@ -492,7 +519,8 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
     /* yes this should be kilo, but then the log and pow bits below
        should be base 'kilo' as well and not base 10 */
 
-    while (number >= (double) 1000 && digits < (long long) kMGnumber * 3)
+    while (digits < (long long) kMGnumber * 3 &&
+           (number >= (double) 1000 || short_si[digits / 3][0] == '-'))
       {
 	number /= (double) 1000;
 	digits += 3;
@@ -506,43 +534,24 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
       (pow ((double) 10., (double) (int) (log10 ((double) number))));
   }
 
-  if (kMG)
-    {
-      int count = (int) digits;
-      short_si_out = kMGcopy;	/* JML bugfix */
-      strcpy (kMGcopy, kMG);	/* JML bugfix */
-      while (count >= 3)
-	{
-	  count -= 3;
-	  short_si_out = strchr (short_si_out, ',') + 1;
-	}
-      if (strchr (short_si_out, ','))
-	{
-	  *((char *) strchr (short_si_out, ',')) = '\0';
-	}
-    }
-  else
-    {
-      short_si_out = short_si[min ((signed) (digits / 3), kMGnumber)];
-    }
+  short_si_out = short_si[min ((signed) (digits / 3), kMGnumber)];
 
   if (maxv_q * yticsf >= 1)
     {
-/*    	digits = log10((double)maxv_q); */
-      digits = log10 ((double) maxv_q * yticsf);
+      digits1 = log10 ((double) maxv_q * yticsf);
     }
   else
     {
-      digits = 0;
+      digits1 = 0;
     }
 
 /*  sca_ten = maxv_q / pow(10.0,(double)digits); */
 /*  sca_hun = maxv_q / pow(10.0,(double)digits-1); */
-  sca_ten = (double) maxv_q *yticsf / pow (10.0, (double) digits);
-  sca_hun = (double) maxv_q *yticsf / pow (10.0, (double) digits - 1);
-  nex_rnd = (sca_hun) * pow (10.0, (double) digits - 1);
-  nex_ten = (sca_ten + 1) * pow (10.0, (double) digits);
-  nex_hun = (sca_hun + 1) * pow (10.0, (double) digits - 1);
+  sca_ten = (double) maxv_q *yticsf / pow (10.0, (double) digits1);
+  sca_hun = (double) maxv_q *yticsf / pow (10.0, (double) digits1 - 1);
+  nex_rnd = (sca_hun) * pow (10.0, (double) digits1 - 1);
+  nex_ten = (sca_ten + 1) * pow (10.0, (double) digits1);
+  nex_hun = (sca_hun + 1) * pow (10.0, (double) digits1 - 1);
 
 /*  if (nex_ten <= (1.1 * maxv_q))  { */
   if (nex_ten <= (1.1 * maxv_q * yticsf))
@@ -560,7 +569,7 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
     }
 
   sca_max_q = nmax_q / (pow (10.0, (double) ((int) (digits / 3)) * 3));
-/*  nmax=sca_max_q*ytics*(pow((double)kilo,(double)((int)(digits/3))));  */
+/*  nmax=sca_max_q*ytics*(pow((double)kilo,(double)((int)(digits1/3))));  */
   nmax =
     (sca_max_q / yticsf) * ytics *
     (pow ((double) kilo, (double) ((int) (digits / 3))));
@@ -732,7 +741,7 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
   origmaxvo = (origmaxvo * muli / nmax ) * maxy;
 
   /* Log and second-mean scaling added by Benjamin Despres, 2004-10-13 */
-  if ((options & OPTION_LOGGRAPH) || (options & OPTION_MEANOVER))
+  if (options & OPTION_MEANOVER)
     {
       lmx1 = lmx2 = mea1 = mea2 = temp = 0.0;
       for (x = 0; x < maxx; x++)
@@ -763,8 +772,6 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 	origmaxvo = 1.0;
 
       mea1 /= (double) maxx;
-      if (options & OPTION_MEANOVER)
-	{
 	  for (x = 0; x < maxx; x++)
 	    {
 	      y =
@@ -788,19 +795,6 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 	      if (lhist[x].outmax > mea2)
 		lhist[x].outmax = mea2;
 	    }
-	}
-      if (options & OPTION_LOGGRAPH)
-	{
-	  for (x = 0; x < maxx; x++)
-	    {
-	      lhist[x].in = log (lhist[x].in);
-	      lhist[x].out = log (lhist[x].out);
-	      lhist[x].inmax = log (lhist[x].inmax);
-	      lhist[x].outmax = log (lhist[x].outmax);
-	    }
-	  origmaxvi = log (origmaxvi);
-	  origmaxvo = log (origmaxvo);
-	}
       for (x = 0; x < maxx; x++)
 	{
 	  if (lhist[x].in > lmx2)
@@ -824,9 +818,20 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
       origmaxvi *= lmx2;
       origmaxvo *= lmx2;
 
-      if (options & OPTION_MEANOVER)
-	sca_max_q *= ((float) mea2 / (float) maxy);
-    }				/* end of primary log and second-mean scaling code (more below) */
+      sca_max_q *= ((float) mea2 / (float) maxy);
+    }
+  else if (options & OPTION_LOGGRAPH)
+    {
+      for (x = 0; x < maxx; x++)
+        {
+          lhist[x].in = logscale (lhist[x].in, maxy);
+          lhist[x].out = logscale (lhist[x].out, maxy);
+          lhist[x].inmax = logscale (lhist[x].inmax, maxy);
+          lhist[x].outmax = logscale (lhist[x].outmax, maxy);
+        }
+      origmaxvi = logscale (origmaxvi, maxy);
+      origmaxvo = logscale (origmaxvo, maxy);
+    }	/* end of primary log and second-mean scaling code (more below) */
 
   /* the graph is made ten pixels higher to acomodate the x labels */
   graph = gdImageCreate (XSIZE, YSIZE);
@@ -999,34 +1004,6 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 		       (unsigned char *) longup, i_grid);
     }
 
-  /* secondary log and second-mean scale code block follows */
-  if (options & OPTION_LOGGRAPH)
-    {
-      for (i = 0; i <= ytics; i++)
-	{
-	  gdImageLine (graph, xtr (-2), ytr (i * maxy / ytics), xtr (1),
-		       ytr (i * maxy / ytics), i_grid);
-	  gdImageLine (graph, xtr (maxx + 2), ytr (i * maxy / ytics),
-		       xtr (maxx - 1), ytr (i * maxy / ytics), i_grid);
-	  gdImageLine (graph, xtr (0), ytr (i * maxy / ytics), xtr (maxx),
-		       ytr (i * maxy / ytics), gdStyled);
-	  temp = exp ((log ((float) maxy) / (float) ytics) * (float) i) - 1.0;
-	  temp *= ((float) sca_max_q * (float) ytics) / (float) maxy;
-	  sprintf (ylab, "%6.1f %s", temp, short_si_out);
-	  gdImageString (graph, gdFontSmall, 23,
-			 ytr (i * maxy / ytics + gdFontSmall->h / 2),
-			 (unsigned char *) ylab, i_grid);
-	  if (options & OPTION_DORELPERCENT)
-	    {
-	      sprintf (ylab, "% 6.1f%%", (float) 100 / ytics * i);
-	      gdImageString (graph, gdFontSmall, 77 + ((maxx) * xscale) + 1,
-			     ytr (i * maxy / ytics + gdFontSmall->h / 2),
-			     (unsigned char *) ylab, i_outpg);
-	    }
-	}
-    }
-  else				/* end of secondary log and second-mean scaling code */
-    {
       for (i = 0; i <= ytics; i++)
 	{
 
@@ -1043,7 +1020,12 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
     	sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si[digits/3]);
 */
 /*    	sprintf(ylab,"%6.1f %s",sca_max_q*i*yticsf,short_si_out);  */
-	  sprintf (ylab, "%6.1f %s", sca_max_q * i, short_si_out);
+	  temp = sca_max_q * i;
+	  if (options & OPTION_LOGGRAPH)
+	     temp = expscale(maxy * i / ytics, maxy) * ytics * sca_max_q / maxy;
+	  else
+	     temp = sca_max_q * i;
+	  sprintf (ylab, "%6.1f %s", temp, short_si_out);
 
 /*        sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si_out); */
 	  gdImageString (graph, gdFontSmall, 23,
@@ -1053,13 +1035,12 @@ image (file, maxvi, maxvo, maxx, maxy, xscale, yscale, growright, step, bits,
 	  if (options & OPTION_DORELPERCENT)
 	    {
 	      /* sprintf(ylab,"% 6.1f%%",(float)25.*i); */
-	      sprintf (ylab, "% 6.1f%%", (float) 100 / ytics * i);
+	      sprintf (ylab, "% 6.1f%%", (float) (temp / (sca_max_q * ytics) * 100));
 	      gdImageString (graph, gdFontSmall, 77 + ((maxx) * xscale) + 1,
 			     ytr (i * maxy / ytics + gdFontSmall->h / 2),
 			     (unsigned char *) ylab, i_outpg);
 	    }
 	}
-    }
 
   /* draw vertical grid and horizontal labels */
   for (x = 0; x < maxx; x++)
@@ -2193,7 +2174,6 @@ main (argc, argv)
 	case 'K':
 	  kMG = calloc (strlen (argv[argi + 1]) + 1, sizeof (char));
 	  strcpy (kMG, argv[argi + 1]);
-	  kMGcopy = calloc (strlen (argv[argi + 1]) + 1, sizeof (char));
 	  used = 2;
 	  break;
 	case 'Z':		/* Timezone name */
